# *        heim           *
# * Author: Semyon Sokolov *

import json
import sys
from pathlib import Path

from rich.text import Text

from textual import on, work
from textual.app import App, ComposeResult, Binding
from textual.validation import Validator, ValidationResult
from textual.widgets import Footer, Tree, Button
from textual.widgets.tree import TreeNode

from .custom_widgets.top_bar import TopBar
from .platform_functions.directory_generator import DirectoryGenerator

from .screens.main_menu_screen import MainMenuScreen
from .screens.load_json_screen import LoadJSONScreen
from .screens.export_json_screen import ExportJSONScreen
from .screens.new_root_screen import NewRootScreen
from .screens.node_rename_screen import NodeRenameScreen
from .screens.export_json_name_screen import ExportJSONNameScreen
from .screens.overwrite_json import OverwriteJSONScreen


class ViablePathValidator(Validator):
    """Checks whether the input conforms to the os rules for paths"""

    def validate(self, value: str) -> ValidationResult:
        path = Path(value)
        if path.resolve(strict=False) and "/" not in value and value != "/":
            return self.success()
        else:
            return self.failure()


class HeimApp(App):
    """A terminal tool for creating directory trees from your presets"""

    BINDINGS = [
        Binding("ctrl+l", "load_tree", "Load a preset", show=True),
        Binding("ctrl+s", "save_tree", "Save a preset"),
        Binding("ctrl+e", "execute_tree", "Execute a preset"),
        Binding("a", "add_node", "Add Folder", show=True),
        Binding("r", "rename", "Rename Folder", show=True),
        Binding("ctrl+r", "new_root", "New Root", show=True),
        Binding("d", "delete_node", "Delete Folder", show=True),
        Binding("ctrl+d", "delete_node_children", "Delete Folders Inside", show=True),
        Binding("ctrl+n", "new_tree", "New Tree", show=False),
        Binding("k", "show_help_panel()", "Show Help panel", show=False),
        Binding("l", "hide_help_panel()", "Hide Help panel", show=False),
        Binding("t", "change_theme", "Change app theme", show=False),
        Binding("Esc", "none", "Close menu", show=False),
    ]
    TITLE = "~heim~"
    CSS_PATH = "heim.css"

    def __init__(self):
        self.tree_label = Path.cwd() if len(sys.argv) < 2 else sys.argv[1]
        super().__init__()

    def compose(self) -> ComposeResult:
        """Yields widgets for the app"""
        yield TopBar()
        yield Tree(str(self.tree_label))
        yield Footer()

    def on_mount(self) -> None:
        self.query_one(Tree).focus()

    # CLASS METHODS

    @classmethod
    def add_json(cls, node: TreeNode, json_data: object) -> None:
        """Creates the entire node tree via a loop"""

        def add_node(name: str, node: TreeNode, data: object) -> None:
            """Checks the input data object type and assigns a label to an empty node, starting a new loop"""

            node.set_label(Text(f"{name}"))
            node.expand()

            if isinstance(data, dict):
                for key, value in data.items():
                    new_node = node.add("")
                    add_node(key, new_node, value)
            elif isinstance(data, list):
                for index, value in enumerate(data):
                    new_node = node.add("")
                    add_node(str(index), new_node, value)
            else:
                node.allow_expand = False
                label = Text(str(data))
                node.set_label(label)

        add_node(HeimApp().tree_label, node, json_data)

    @classmethod
    def tree_to_json(cls, node: TreeNode):

        def has_grandchildren(node: TreeNode):
            """Check if a node has any grandchildren"""
            return any(child.children for child in node.children)

        def tree_to_dict(node: TreeNode):
            if has_grandchildren(node):
                return {
                    str(child.label): tree_to_dict(child) for child in node.children
                }
            else:
                return [str(child.label) for child in node.children]

        tree_dict = tree_to_dict(node)
        return tree_dict

    # BUTTONS & ACTIONS

    @on(Button.Pressed, "#main_menu")
    def open_menu(self):
        """Action to display the main menu screen"""
        self.push_screen(MainMenuScreen())

    @on(Button.Pressed, "#load_tree")
    @work
    async def action_load_tree(self):
        """Mounts a Load JSON screen when 'Load' button is pressed"""
        file_path = await self.push_screen_wait(LoadJSONScreen())
        with open(file_path) as data_file:
            self.json_data = json.load(data_file)
        tree = self.query_one(Tree)
        tree.root.remove_children()
        self.add_json(tree.root, self.json_data)
        tree.root.expand()

    @on(Button.Pressed, "#save_tree")
    @work
    async def action_save_tree(self):
        """Mounts a Export JSON screen when 'Save' button is pressed"""
        file_path = await self.push_screen_wait(ExportJSONScreen())
        file_name = await self.push_screen_wait(ExportJSONNameScreen())
        complete_preset_path = file_path / file_name
        tree = self.query_one(Tree)
        tree_to_json = self.tree_to_json(tree.root)

        def write_file(data: dict):
            with open(complete_preset_path, "w", encoding="utf-8") as json_file:
                try:
                    json.dump(
                        data,
                        json_file,
                        sort_keys=True,
                        indent=4,
                        ensure_ascii=False,
                    )
                except TypeError as e:
                    raise ValueError(f"Data serialization failed: {e}") from None

        if complete_preset_path.exists():
            overwrite_choice = await self.push_screen_wait(OverwriteJSONScreen())
            if not overwrite_choice:
                self.notify("The preset was not saved", severity="error", timeout=5)
            else:
                # pass
                write_file(tree_to_json)
        else:
            write_file(tree_to_json)

    def get_node_offset(self):
        tree = self.query_one(Tree)
        scroll_offset_y = tree.scroll_offset.y
        scroll_offset_x = tree.scroll_offset.x
        tree_offset_y = self.screen.get_offset(tree).y
        tree_offset_x = self.screen.get_offset(tree).x
        cursor_node = tree.cursor_node
        node_line = cursor_node.line
        node_offset_y = tree_offset_y + node_line - scroll_offset_y

        def get_node_depth(node):
            depth = 4
            if node != tree.root:
                while node.parent != tree.root:
                    depth += 4
                    node = node.parent
            return depth

        node_depth = tree_offset_x + get_node_depth(cursor_node) - scroll_offset_x
        return [node_offset_y, node_depth]

    @on(Button.Pressed, "#new_root")
    @work
    async def action_new_root(self) -> None:
        """Action to display the new root input screen"""
        new_root = await self.push_screen_wait(NewRootScreen())
        target_tree = self.query_one(Tree)
        target_tree.root.set_label(str(new_root))

    @on(Button.Pressed, "#execute_tree")
    @work
    async def action_execute_tree(self) -> None:
        tree = self.query_one(Tree)
        generated_tree = DirectoryGenerator(tree)
        if generated_tree.generation_status:
            self.notify(
                "The tree was successfully generated!",
                severity="information",
                timeout=7,
            )
        elif not generated_tree.generation_status:
            self.notify(
                generated_tree.generation_status_error, severity="error", timeout=7
            )

    def action_new_tree(self) -> None:
        """Clears the node tree"""
        tree = self.query_one(Tree)
        tree.clear()

    def action_add_node(self):
        """Adds a folder to the tree"""
        tree = self.query_one(Tree)
        tree.cursor_node.add("folder")
        tree.cursor_node.expand()

    def action_delete_node(self):
        """Deletes a folder from the tree"""
        tree = self.query_one(Tree)
        if not tree.cursor_node.is_root:
            tree.cursor_node.remove()

    def action_delete_node_children(self):
        """Deletes a folder's children from the tree"""
        tree = self.query_one(Tree)
        tree.cursor_node.remove_children()
        tree.cursor_node.collapse()

    @work
    async def action_rename(self):
        """Renames the selected folder"""
        tree = self.query_one(Tree)
        selected_node = tree.cursor_node
        node_offset = self.get_node_offset()
        if not selected_node.is_root:
            siblings = selected_node.siblings
            new_node_name = await self.push_screen_wait(
                NodeRenameScreen(str(tree.cursor_node.label), siblings, node_offset)
            )
            selected_node.set_label(new_node_name)


if __name__ == "__main__":
    app = HeimApp()
    app.run()
