import psutil
import re

from textual import events
from textual.app import ComposeResult
from textual.widgets import Input
from textual.screen import ModalScreen
from textual.validation import Validator, ValidationResult


class ViableNameValidator(Validator):
    def validate(self, value: str) -> ValidationResult:
        if psutil.WINDOWS:
            forbidden_chars = r'[<>:"/\\|?*]'
            reserved_names = [
                "CON",
                "PRN",
                "AUX",
                "NUL",
                "COM1",
                "COM2",
                "COM3",
                "COM4",
                "COM5",
                "COM6",
                "COM7",
                "COM8",
                "COM9",
                "COM0",
                "LPT1",
                "LPT2",
                "LPT3",
                "LPT4",
                "LPT5",
                "LPT6",
                "LPT7",
                "LPT8",
                "LPT9",
                "LPT0",
            ]
            if re.search(forbidden_chars, value):
                return self.failure("Forbidden Character!")
            elif value.upper() in reserved_names:
                return self.failure("Reserved Name!")
            elif len(value) > 255:  # length limit for most common OS
                return self.failure("Name is too long!")
            elif value == "":
                return self.failure("Empty Input!")
            else:
                return self.success()
        else:
            # name validation for LINUX
            forbidden_chars = r"/"
            if re.search(forbidden_chars, value):
                return self.failure("Forbidden Character!")
            elif len(value) > 255:  # length limit for most common OS
                return self.failure("Name is too long!")
            elif value == "." or value == "..":
                return self.failure("Reserved Name!")
            elif value == "":
                return self.failure("Empty Input!")
            else:
                return self.success()


class ExportJSONNameScreen(ModalScreen):
    """Exports a name for the .json preset"""

    def compose(self) -> ComposeResult:
        yield Input(
            placeholder="preset name",
            id="preset_name_input",
            type="text",
            validators=ViableNameValidator(),
            valid_empty=False,
        )

    def on_mount(self) -> None:
        self.title = "Preset name"

    def on_input_submitted(self, event: Input.Submitted):
        """Gets the name for the preset"""
        # preset_name = event.value
        if event.validation_result.is_valid:
            if not event.value.endswith(".json"):
                preset_name_json = event.value + ".json"
                self.dismiss(preset_name_json)
            else:
                self.dismiss(event.value)
        else:
            return self.notify(
                event.validation_result.failure_descriptions[0],
                severity="error",
                timeout=5,
            )

    def on_key(self, event: events.Key) -> None:
        """Closes the Preset name screen"""
        if event.key == "escape":
            self.app.pop_screen()
