from typing import Iterable

import psutil
import sys
import re
from pathlib import Path

from textual import events, work
from textual.app import ComposeResult
from textual.binding import Binding
from textual.widgets import DirectoryTree, Header, Tree, Input
from textual.screen import ModalScreen
from textual.containers import Grid
from textual.validation import Validator, ValidationResult

from ..platform_functions.get_drives import GetDrives


class ViablePathValidator(Validator):
    def validate(self, value: str) -> ValidationResult:
        path = Path(value)
        try:
            if value != "":
                if path.exists() & path.is_dir():
                    return self.success()
                else:
                    return self.failure(
                        "The path should lead to an existing directory!"
                    )
            else:
                return self.failure("Empty input")
        except Exception as e:
            return self.failure(f"unexpected error occurred: {e}")


class DirectoryTreeModified(DirectoryTree):
    # PATH FILTER, RETURNS ONLY DIR NAMES AND .JSON FILES
    def filter_paths(self, paths: Iterable[Path]) -> Iterable[Path]:
        path_list = []
        for path in paths:
            if path.is_dir() or path.name.endswith(".json"):
                path_list.append(path)
        return path_list


class ExportJSONScreen(ModalScreen):
    """Screen for exporting a .json preset"""

    BINDINGS = [
        Binding("space", "toggle_node", "Expand a directory"),
        Binding("enter", "select_cursor", "Select a directory"),
    ]
    root_label = Path.cwd() if len(sys.argv) < 2 else sys.argv[1]

    def compose(self) -> ComposeResult:

        with Grid(id="export_grid"):
            yield Header()
            yield Tree("Drives", id="tree_new")
            yield DirectoryTreeModified(
                self.root_label,
                id="directory_tree_modified",
            ).focus()
            yield Input(
                placeholder="input a path",
                id="export_path_input",
                type="text",
                validators=ViablePathValidator(),
                valid_empty=False,
            )

    def on_mount(self) -> None:
        self.add_drives()
        self.title = "Save to"
        self.sub_title = '"space" to expand, "enter" to select'

    def add_drives(self) -> None:
        self.drives = GetDrives().drives
        tree = self.query_one(Tree)
        if psutil.WINDOWS:
            for drive in self.drives:
                label = f"{drive['label']} ({drive['device']})"
                tree.root.add_leaf(label)
        elif psutil.LINUX:
            for drive in self.drives:
                tree.root.add_leaf(drive["label"])
        tree.root.expand()
        tree.show_root = False

    def on_directory_tree_directory_selected(
        self, event: DirectoryTreeModified.DirectorySelected
    ):
        self.dismiss(event.path)

    @work
    async def on_tree_node_selected(self, event: Tree.NodeSelected):
        new_drive = ""
        if psutil.WINDOWS:
            s = str(event.node.label)
            match = re.search(r"\((.*?)\)", s)
            if match:
                new_drive = match.group(1)
        else:
            new_drive = str(event.node.label)

        self.root_label = new_drive
        await self.recompose()
        self.add_drives()
        dir_tree = self.query_one(DirectoryTreeModified)
        dir_tree.focus()

    def on_input_submitted(self, event: Input.Submitted):
        if event.validation_result.is_valid:
            path_to_dir = Path(str(event.value))
            self.dismiss(path_to_dir)
        else:
            self.notify(
                event.validation_result.failure_descriptions[0],
                severity="error",
                timeout=5,
            )

    def on_key(self, event: events.Key) -> None:
        """Closes the Load screen"""
        if event.key == "escape":
            self.app.pop_screen()
