import sys
import re
from typing import Iterable

import psutil

from pathlib import Path

from textual import events, work
from textual.app import ComposeResult
from textual.widgets import DirectoryTree, Tree, Input
from textual.screen import ModalScreen
from textual.containers import Grid
from textual.reactive import reactive
from textual.validation import Validator, ValidationResult

from ..platform_functions.get_drives import GetDrives


class ViableJSONValidator(Validator):
    def validate(self, value: str) -> ValidationResult:
        path = Path(value)
        # if path.is_file():
        try:
            if path.is_file():
                if path.suffix == ".json":
                    return self.success()
                else:
                    return self.failure("The file must have '.json' extension!")
            elif path.is_dir():
                return self.failure("The path must lead to a '.json' file!")
            # elif path.suffix != '.json':
            #     return self.failure("The file must have '.json' extension!")
            else:
                return self.failure("Input must be a path to '.json' file!")
        except Exception as e:
            return self.failure(f"An unexpected error has occurred: {e}")


class FilteredDirectoryPath(DirectoryTree):

    def filter_paths(self, paths: Iterable[Path]) -> Iterable[Path]:
        return [path for path in paths if path.name.endswith(".json") or path.is_dir()]


class LoadJSONScreen(ModalScreen):
    """Screen for choosing a .json preset"""

    # dir_tree_path = reactive("./")

    root_label = Path.cwd() if len(sys.argv) < 2 else sys.argv[1]

    def compose(self) -> ComposeResult:
        # root_label = Path.cwd() if len(sys.argv) < 2 else sys.argv[1]
        # drives = GetDrives().drives
        # root_label = str(drives[0['label']])
        with Grid(id="load_grid"):
            yield Tree("Drives", id="drive_list")
            yield FilteredDirectoryPath(self.root_label, id="dir_list").focus()
            yield Input(
                placeholder="input a path",
                id="path_input",
                type="text",
                validators=ViableJSONValidator(),
                valid_empty=False,
            )

    def on_mount(self) -> None:
        self.add_drives()

    def add_drives(self) -> None:
        self.drives = GetDrives().drives
        tree = self.query_one(Tree)
        if psutil.WINDOWS:
            for drive in self.drives:
                label = f"{drive['label']} ({drive['device']})"
                tree.root.add_leaf(label)
        elif psutil.LINUX:
            for drive in self.drives:
                tree.root.add_leaf(drive["label"])
        tree.root.expand()
        tree.show_root = False

    # @on(Input.Submitted, "#path_input")
    # def load_json_data(self) -> None:
    def on_input_submitted(self, event: Input.Submitted):
        if event.validation_result.is_valid:
            self.dismiss(event.value)
        else:
            self.notify(
                event.validation_result.failure_descriptions[0],
                severity="error",
                timeout=5,
            )

    def on_directory_tree_file_selected(self, event: DirectoryTree.FileSelected):
        self.dismiss(event.path)

    # def on_node_selected(self, event:Tree.NodeSelected):
    #     dir_tree = self.query_one(DirectoryTree).

    @work
    async def on_tree_node_selected(self, event: Tree.NodeSelected):
        new_drive = ""
        if psutil.WINDOWS:
            s = str(event.node.label)
            match = re.search(r"\((.*?)\)", s)
            if match:
                new_drive = match.group(1)
        else:
            new_drive = str(event.node.label)

        # new_drive = str(event.node.label)
        self.root_label = new_drive
        await self.recompose()
        self.add_drives()
        dir_tree = self.query_one(FilteredDirectoryPath)
        dir_tree.focus()

    def on_key(self, event: events.Key) -> None:
        """Closes the Load screen"""
        if event.key == "escape":
            self.app.pop_screen()

    # async def on_tree_node_selected(self, event: Tree.NodeSelected) -> None:
    #     node_data = event.node.data
    #     dir_tree = self.query_one(DirectoryTree)
    #     dir_tree.path = node_data
    #
