import re
import psutil

from textual import events
from textual.app import ComposeResult
from textual.screen import ModalScreen
from textual.validation import Validator, ValidationResult
from textual.widgets import Input


class ViablePathValidator(Validator):
    """Checks whether the input conforms to the os rules for paths"""

    def __init__(self, siblings):
        self.siblings = siblings
        self.sibling_name = [str(sibling.label) for sibling in self.siblings]
        super().__init__()

    def validate(self, value: str) -> ValidationResult:
        if psutil.WINDOWS:
            forbidden_chars = r'[<>:"/\\|?*]'
            reserved_names = [
                "CON",
                "PRN",
                "AUX",
                "NUL",
                "COM1",
                "COM2",
                "COM3",
                "COM4",
                "COM5",
                "COM6",
                "COM7",
                "COM8",
                "COM9",
                "COM0",
                "LPT1",
                "LPT2",
                "LPT3",
                "LPT4",
                "LPT5",
                "LPT6",
                "LPT7",
                "LPT8",
                "LPT9",
                "LPT0",
            ]
            if re.search(forbidden_chars, value):
                return self.failure("Forbidden Character!")
            elif value.upper() in reserved_names:
                return self.failure("Reserved Name!")
            elif len(value) > 255:  # length limit for most common OS
                return self.failure("Name is too long!")
            elif value in self.sibling_name:  # checks for siblings
                return self.failure("This name is already taken in this sub-directory!")
            elif value == "":
                return self.failure("Empty Input!")
            else:
                return self.success()
        else:
            forbidden_chars = r"/"
            if re.search(forbidden_chars, value):
                return self.failure("Forbidden Character!")
            elif len(value) > 255:  # length limit for most common OS
                return self.failure("Name is too long!")
            elif value in self.sibling_name:  # checks for siblings
                return self.failure("This directory already exists in your graph!")
            elif value == "":
                return self.failure("Empty Input!")
            else:
                return self.success()


class NodeRenameScreen(ModalScreen[str]):
    """Screen for changing a node's name"""

    def __init__(self, current_name, siblings, node_data):
        self.current_name = current_name
        self.node_siblings = siblings
        self.node_offset_y = node_data[0]
        self.node_offset_x = node_data[1]
        super().__init__()

    def compose(self) -> ComposeResult:
        yield Input(
            self.current_name,
            select_on_focus=True,
            id="new_name",
            validators=[ViablePathValidator(self.node_siblings)],
            valid_empty=False,
        ).focus()

    def on_mount(self):
        input = self.query_one(Input)
        input.styles.offset = (self.node_offset_x, self.node_offset_y)

    def on_key(self, event: events.Key) -> None:
        """Closes the screen"""
        if event.key == "escape":
            self.app.pop_screen()

    def on_input_submitted(self, event: Input.Submitted):
        if event.validation_result.is_valid:
            self.dismiss(event.value)
        else:
            self.notify(
                event.validation_result.failure_descriptions[0],
                severity="error",
                timeout=5,
            )
